#include "bulletmlparser.h"
#include "bulletmlparser-ygg.h"
#include "bulletmlerror.h"
#include "bulletmltree.h"

#include "ygg/ygg.h"

#include <string>

using namespace yggdrasil;

BulletMLParserYggdrasil::BulletMLParserYggdrasil(const std::string& filename)
    : xmlFile_(filename), curNode_(0)
{
	setName(filename);
}

BulletMLParserYggdrasil::~BulletMLParserYggdrasil() {}

void BulletMLParserYggdrasil::getTree(ygg_node node) {
	translateNode(node);

	ygg_node text = node[node.get_path()+"/text()"];
	for (int i = 0; text[i].is_valid(); i++) {
		curNode_->setValue(trimString(text[i].get_value()));
	}

    ygg_node child = node[node.get_path()+"/element()"];
	for (int i = 0; child[i].is_valid(); i++) {
		getTree(child[i]);
    }

    curNode_ = curNode_->getParent();
}

void BulletMLParserYggdrasil::translateNode(ygg_node node) {
    BulletMLNode* xmlNode = addContent(trimString(node[0].get_name()));

	if (xmlNode->getName() != BulletMLNode::bulletml) {
		MyAttributes mattr;
		ygg_node attr = node[node.get_path()+"/attribute()"];
		for (int i = 0; attr[i].is_valid(); i++) {
			mattr.push_back(trimString(attr[i].get_name()));
			mattr.push_back(trimString(attr[i].get_value()));
		}
		addAttribute(mattr, xmlNode);
		if (curNode_ != 0) curNode_->addChild(xmlNode);
	}
	curNode_ = xmlNode;
}

void BulletMLParserYggdrasil::parse() {
	sox_file root(xmlFile_);
	root.read();

	ygg_node type = root["/bulletml@type"];
	for (int i = 0; type[i].is_valid(); i++) {
		if (type[i].get_value() == "horizontal") setHorizontal();
	}

	getTree(root["/bulletml"]);
}

std::string BulletMLParserYggdrasil::trimString(const std::string& str) {
	std::string ret(str);

	if (!ret.empty() && ret[0] == ' ')
		ret.erase(0, ret.find_first_not_of(' '));
	if (!ret.empty() && ret[str.length()-1] == ' ')
		ret.erase(ret.find_last_not_of(' ') + 1);

	return ret;
}

